"use strict";

var _interopRequireDefault = require("reshow-runtime/helpers/interopRequireDefault");
var _interopRequireWildcard = require("reshow-runtime/helpers/interopRequireWildcard");
exports.__esModule = true;
exports["default"] = void 0;
var _objectDestructuringEmpty2 = _interopRequireDefault(require("reshow-runtime/helpers/objectDestructuringEmpty"));
var _extends2 = _interopRequireDefault(require("reshow-runtime/helpers/extends"));
var _objectSpread2 = _interopRequireDefault(require("reshow-runtime/helpers/objectSpread2"));
var _objectWithoutPropertiesLoose2 = _interopRequireDefault(require("reshow-runtime/helpers/objectWithoutPropertiesLoose"));
var _getStorage = require("get-storage");
var _getObjectValue = _interopRequireWildcard(require("get-object-value"));
var _setObjectValue = _interopRequireDefault(require("set-object-value"));
var _seturl = require("seturl");
var _callFunc = _interopRequireDefault(require("call-func"));
var _reshowConstant = require("reshow-constant");
var _getParams5 = _interopRequireDefault(require("./getParams.js"));
var _getTime = _interopRequireDefault(require("./getTime.js"));
var _excluded = ["stop", "wait", "lazeInfo", "lazyKey"]; // @ts-check
var lazyActionKey = "lazyAction";
var PARAMS = "params";
var hashKey = "__hash";
var seqKey = "__seq";

/**
 * @template StateType
 * @template ActionType
 * @typedef {import("reshow-flux-base").DispatchFunction<StateType, ActionType>} DispatchFunction
 */
/**
 * @typedef {Object<sinon,any>} StateType
 */
/**
 * @typedef {import("reshow-flux-base").ActionObject} ActionObject
 */

/**
 * @template StateType
 * @template ActionType
 * @param {any} lazyAction
 * @param {DispatchFunction<StateType, ActionType>} dispatch
 */
var processLazyAction = function processLazyAction(lazyAction, dispatch) {
  /**
   * @param {any} lazeArr
   * @param {number|string} key
   */
  var processLazy = function processLazy(lazeArr, key) {
    var laze = lazeArr[key];
    var _getParams = (0, _getParams5["default"])(laze),
      wait = _getParams.wait,
      stop = _getParams.stop;
    if (!wait || wait <= 0) {
      if (!stop) {
        if (_reshowConstant.UNDEFINED !== typeof (0, _getObjectValue["default"])(laze, ["params", "wait"])) {
          delete laze.params.wait;
        }
        dispatch && dispatch(laze);
      }
      delete lazeArr[key];
    } else {
      laze.params.wait = wait - 1;
    }
    return lazeArr[key];
  };
  var seq = (0, _getObjectValue["default"])(lazyAction, [seqKey]);
  if ((0, _reshowConstant.IS_ARRAY)(seq)) {
    lazyAction.__seq = seq.filter(function (_action, key) {
      return processLazy(seq, key);
    });
  }
  var hash = (0, _getObjectValue["default"])(lazyAction, [hashKey]);
  if (hash) {
    (0, _reshowConstant.KEYS)(hash).forEach(function (key) {
      return processLazy(hash, key);
    });
  }
  return lazyAction;
};
var getDefaultStorage = function getDefaultStorage() {
  return new _getStorage.Storage(_getStorage.localStorage);
};

/**
 * @param {Storage} storage
 */
var initLazyAction = function initLazyAction(storage) {
  storage = storage || getDefaultStorage();
  var getAllLazy = function getAllLazy() {
    return (0, _getObjectValue.toMap)(storage.get(lazyActionKey));
  };
  var getOneLazy = function getOneLazy( /**@type string*/k) {
    return (0, _getObjectValue.toMap)(getAllLazy().__hash)[k];
  };
  var updateLazy = function updateLazy( /**@type any*/lazyAction) {
    return storage.set(lazyActionKey, lazyAction);
  };
  var removeLazy = function removeLazy( /**@type string*/key) {
    var lazyAction = getAllLazy();
    if ((0, _getObjectValue["default"])(lazyAction, [hashKey, key])) {
      delete lazyAction.__hash[key];
      updateLazy(lazyAction);
    }
  };

  /**
   * @param {any} action
   * @param {string} key
   */
  var getActionMergeWithLazy = function getActionMergeWithLazy(action, key) {
    var lazyAction = getAllLazy();
    var _get = (0, _getObjectValue["default"])(lazyAction, [hashKey, key, PARAMS], {}),
      stop = _get.stop,
      wait = _get.wait,
      lazeInfo = _get.lazeInfo,
      lazyKey = _get.lazyKey,
      lazeParams = (0, _objectWithoutPropertiesLoose2["default"])(_get, _excluded);
    (0, _reshowConstant.KEYS)(lazeParams).forEach(function (pKey) {
      var p = lazeParams[pKey];
      var newP = _reshowConstant.OBJECT === typeof p ? (0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, p), (0, _getObjectValue["default"])(action, [PARAMS, pKey], {})) : (0, _getObjectValue["default"])(action, [PARAMS, pKey], p);
      (0, _setObjectValue["default"])(action, [PARAMS, pKey], newP);
    });
    delete action.params.withLazy;
    return action;
  };

  /**
   * @param {any} action
   * @param {string} key
   */
  var pushLazyAction = function pushLazyAction(action, key) {
    var _getParams2 = (0, _getParams5["default"])(action),
      params = (0, _extends2["default"])({}, ((0, _objectDestructuringEmpty2["default"])(_getParams2), _getParams2));
    var thisAction = {
      params: params,
      type: action.type
    };
    (0, _setObjectValue["default"])(thisAction, [PARAMS, "lazeInfo"], {
      from: (0, _seturl.url)(),
      time: (0, _getTime["default"])().toString()
    });
    var lazyAction = getAllLazy();
    if (key) {
      (0, _setObjectValue["default"])(lazyAction, [hashKey, key], thisAction);
    } else {
      (0, _setObjectValue["default"])(lazyAction, [seqKey], thisAction, true);
    }
    updateLazy(lazyAction);
  };

  /**
   * @template StateType
   * @template ActionType
   * @param {DispatchFunction<StateType, ActionType>} dispatch
   */
  var process = function process(dispatch) {
    return updateLazy(processLazyAction(getAllLazy(), dispatch));
  };

  /**
   * @param {StateType} state
   * @param {ActionObject} action
   */
  var handleAction = function handleAction(state, action) {
    var _getParams3 = (0, _getParams5["default"])(action),
      withLazy = _getParams3.withLazy;
    if (withLazy) {
      action = getActionMergeWithLazy(action, withLazy);
    }
    var actionHandler = state.get("lazyActionHandler");
    var next = (0, _callFunc["default"])(actionHandler, [state, action]) || state;
    var _getParams4 = (0, _getParams5["default"])(action),
      wait = _getParams4.wait,
      stop = _getParams4.stop,
      lazyKey = _getParams4.lazyKey; // need locate after next
    if (_reshowConstant.T_NULL == wait && !stop) {
      if (withLazy && withLazy !== lazyKey) {
        removeLazy(withLazy);
      }
    }
    return next;
  };
  return {
    process: process,
    handleAction: handleAction,
    getAll: getAllLazy,
    getOne: getOneLazy,
    push: pushLazyAction
  };
};
var _default = exports["default"] = initLazyAction;
module.exports = exports.default;