"use strict";

var _interopRequireWildcard = require("reshow-runtime/helpers/interopRequireWildcard");
var _interopRequireDefault = require("reshow-runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports["default"] = void 0;
var _classCallCheck2 = _interopRequireDefault(require("reshow-runtime/helpers/classCallCheck"));
var _createClass2 = _interopRequireDefault(require("reshow-runtime/helpers/createClass"));
var _defineProperty2 = _interopRequireDefault(require("reshow-runtime/helpers/defineProperty"));
var _reshowConstant = require("reshow-constant");
var _setObjectValue = _interopRequireDefault(require("set-object-value"));
var _callFunc = _interopRequireDefault(require("call-func"));
var _getParams4 = _interopRequireWildcard(require("../getParams.js"));
//@ts-check

var INITIAL = "init";

/**
 * @template StateType
 * @template ActionType
 * @typedef {import("reshow-flux-base").StoreObject<StateType, ActionType>} StoreObject
 */

/**
 * @template StateType
 * @template ActionType
 * @typedef {import("reshow-flux-base").DispatchFunction<StateType, ActionType>} DispatchFunction
 */

/**
 * @typedef {Object<sinon,any>} StateType
 */
/**
 * @typedef {import("reshow-flux-base").ActionObject} ActionObject
 */
var BaseI13nReducer = /*#__PURE__*/function () {
  function BaseI13nReducer() {
    (0, _classCallCheck2["default"])(this, BaseI13nReducer);
    /**
     * @abstract
     * @type {StoreObject<StateType, ActionObject>}
     */
    (0, _defineProperty2["default"])(this, "store", void 0);
  }
  return (0, _createClass2["default"])(BaseI13nReducer, [{
    key: "mergeMap",
    value:
    /**
     * @abstract
     * @param {StateType} _state
     * @param {object} _paramsMap
     */
    function mergeMap(_state, _paramsMap) {}

    /**
     * @abstract
     * @template StateType
     * @template ActionType
     * @type {DispatchFunction<StateType, ActionType>}
     * @returns {any}
     */
  }, {
    key: "dispatch",
    value: function dispatch(_action, _actionParams) {}

    /**
     * @param {StateType} state
     * @param {ActionObject} _action
     */
  }, {
    key: "sendBeacon",
    value: function sendBeacon(state, _action) {
      return state;
    }

    /**
     * @param {StateType} state
     * @param {ActionObject} action
     */
  }, {
    key: "initDone",
    value: function initDone(state, action) {
      var _this = this;
      var _getParams = (0, _getParams4["default"])(action),
        triggerImpression = _getParams.triggerImpression,
        asyncInit = _getParams.asyncInit;
      var assignState = function assignState( /**@type StateType*/state) {
        return state.set(INITIAL, true).set("nextEmit", INITIAL);
      };
      if (asyncInit) {
        setTimeout(function () {
          return _this.dispatch("impression");
        });
        return assignState(state);
      } else {
        // has customInitHandler
        setTimeout(function () {
          /**
           * Dispatch store parallel data.
           *
           * Need put it inside setTimeout,
           * to avoid conflict with first impression call.
           *
           */
          _this.dispatch(assignState(state));
          setTimeout(function () {
            if (triggerImpression) {
              /**
               * Why need triggerImpression?
               * if u want let initDone call early, but maybe not dispatch impression soon.
               * u could pass triggerImpression to decide when call dispatch impression.
               */
              triggerImpression(function () {
                return _this.dispatch("impression");
              });
            } else {
              _this.dispatch("impression");
            }
          });
        });
        return state;
      }
    }

    /**
     * @param {StateType} state
     * @param {ActionObject} action
     */
  }, {
    key: "processImpression",
    value: function processImpression(state, action) {
      return this.sendBeacon(state, action);
    }

    /**
     * @param {StateType} state
     * @param {ActionObject} action
     */
  }, {
    key: "processAction",
    value: function processAction(state, action) {
      var vpvid = state.get("vpvid");
      if (vpvid) {
        (0, _setObjectValue["default"])(action, [_getParams4.PARAMS, "query", "vpvid"], vpvid);
      }
      return this.sendBeacon(state, action);
    }

    /**
     * @param {StateType} state
     * @param {ActionObject} action
     */
  }, {
    key: "handleInit",
    value: function handleInit(state, action) {
      var customInitHandler = state.get("initHandler");
      var thisInitDone = this.initDone.bind(this);
      if (!customInitHandler) {
        (0, _setObjectValue["default"])(action, [_getParams4.PARAMS, "asyncInit"], true);
      }
      return (0, _callFunc["default"])(customInitHandler || thisInitDone, [state, action, thisInitDone]);
    }

    /**
     * @param {StateType} state
     * @param {ActionObject} action
     */
  }, {
    key: "handleImpression",
    value: function handleImpression(state, action) {
      var _this2 = this;
      var maybeAsyncRun = function maybeAsyncRun( /**@type StateType*/state) {
        var impressionHandler = state.get("impressionHandler");
        var next = (0, _callFunc["default"])(impressionHandler || _this2.processImpression.bind(_this2), [state, action]);
        var _getParams2 = (0, _getParams4["default"])(action),
          stop = _getParams2.stop; // need locate after next
        if (!stop) {
          next = next.set("nextEmit", "impression");
        }
        return next;
      };
      if (!state.get(INITIAL)) {
        return this.handleInit(state, action);
      } else {
        var disableHandleImpression = state.get("disableHandleImpression");
        return disableHandleImpression ? state : maybeAsyncRun(state);
      }
    }

    /**
     * @param {StateType} state
     * @param {ActionObject} action
     */
  }, {
    key: "handleAction",
    value: function handleAction(state, action) {
      var actionHandler = state.get("actionHandler");
      var next = (0, _callFunc["default"])(actionHandler || this.processAction.bind(this), [state, action]);
      var _getParams3 = (0, _getParams4["default"])(action),
        wait = _getParams3.wait,
        stop = _getParams3.stop; // need locate after next
      if (wait == null && !stop) {
        next = next.set("nextEmit", "action");
      }
      return next;
    }

    /**
     * @param {StateType} state
     * @param {ActionObject} action
     */
  }, {
    key: "reduce",
    value: function reduce(state, action) {
      if (state.get("nextEmit")) {
        state = state.set("nextEmit", null);
      }
      switch (action.type) {
        case "impression":
          return this.handleImpression(state, action);
        case "action":
          return this.handleAction(state, action);
        case "config/set":
          return this.mergeMap(state, action.params);
        case "reset":
          /**
           * !!Important!!
           * Keep in mind, always don't reset localstorage
           * It will effect lazy action
           */
          return this.mergeMap(this.store.reset(), action.params);
        default:
          return (0, _reshowConstant.KEYS)(action).length ? this.mergeMap(state, action) : state;
      }
    }
  }]);
}();
var _default = exports["default"] = BaseI13nReducer;
module.exports = exports.default;