import _objectDestructuringEmpty from "reshow-runtime/es/helpers/objectDestructuringEmpty";
import _extends from "reshow-runtime/es/helpers/extends";
import _objectSpread from "reshow-runtime/es/helpers/objectSpread2";
import _objectWithoutProperties from "reshow-runtime/es/helpers/objectWithoutProperties";
var _excluded = ["stop", "wait", "lazeInfo", "lazyKey"];
// @ts-check
import { localStorage, Storage } from "get-storage";
import get, { toMap } from "get-object-value";
import set from "set-object-value";
import { url } from "seturl";
import callfunc from "call-func";
import { T_NULL, UNDEFINED, OBJECT, KEYS, IS_ARRAY } from "reshow-constant";
import getParams from "./getParams.mjs";
import getTime from "./getTime.mjs";
var lazyActionKey = "lazyAction";
var PARAMS = "params";
var hashKey = "__hash";
var seqKey = "__seq";

/**
 * @template StateType
 * @template ActionType
 * @typedef {import("reshow-flux-base").DispatchFunction<StateType, ActionType>} DispatchFunction
 */
/**
 * @typedef {Object<sinon,any>} StateType
 */
/**
 * @typedef {import("reshow-flux-base").ActionObject} ActionObject
 */

/**
 * @template StateType
 * @template ActionType
 * @param {any} lazyAction
 * @param {DispatchFunction<StateType, ActionType>} dispatch
 */
var processLazyAction = function processLazyAction(lazyAction, dispatch) {
  /**
   * @param {any} lazeArr
   * @param {number|string} key
   */
  var processLazy = function processLazy(lazeArr, key) {
    var laze = lazeArr[key];
    var {
      wait,
      stop
    } = getParams(laze);
    if (!wait || wait <= 0) {
      if (!stop) {
        if (UNDEFINED !== typeof get(laze, ["params", "wait"])) {
          delete laze.params.wait;
        }
        dispatch && dispatch(laze);
      }
      delete lazeArr[key];
    } else {
      laze.params.wait = wait - 1;
    }
    return lazeArr[key];
  };
  var seq = get(lazyAction, [seqKey]);
  if (IS_ARRAY(seq)) {
    lazyAction.__seq = seq.filter(function (_action, key) {
      return processLazy(seq, key);
    });
  }
  var hash = get(lazyAction, [hashKey]);
  if (hash) {
    KEYS(hash).forEach(function (key) {
      return processLazy(hash, key);
    });
  }
  return lazyAction;
};
var getDefaultStorage = function getDefaultStorage() {
  return new Storage(localStorage);
};

/**
 * @param {Storage} storage
 */
var initLazyAction = function initLazyAction(storage) {
  storage = storage || getDefaultStorage();
  var getAllLazy = function getAllLazy() {
    return toMap(storage.get(lazyActionKey));
  };
  var getOneLazy = function getOneLazy( /**@type string*/k) {
    return toMap(getAllLazy().__hash)[k];
  };
  var updateLazy = function updateLazy( /**@type any*/lazyAction) {
    return storage.set(lazyActionKey, lazyAction);
  };
  var removeLazy = function removeLazy( /**@type string*/key) {
    var lazyAction = getAllLazy();
    if (get(lazyAction, [hashKey, key])) {
      delete lazyAction.__hash[key];
      updateLazy(lazyAction);
    }
  };

  /**
   * @param {any} action
   * @param {string} key
   */
  var getActionMergeWithLazy = function getActionMergeWithLazy(action, key) {
    var lazyAction = getAllLazy();
    var _get = get(lazyAction, [hashKey, key, PARAMS], {}),
      {
        stop,
        wait,
        lazeInfo,
        lazyKey
      } = _get,
      lazeParams = _objectWithoutProperties(_get, _excluded);
    KEYS(lazeParams).forEach(function (pKey) {
      var p = lazeParams[pKey];
      var newP = OBJECT === typeof p ? _objectSpread(_objectSpread({}, p), get(action, [PARAMS, pKey], {})) : get(action, [PARAMS, pKey], p);
      set(action, [PARAMS, pKey], newP);
    });
    delete action.params.withLazy;
    return action;
  };

  /**
   * @param {any} action
   * @param {string} key
   */
  var pushLazyAction = function pushLazyAction(action, key) {
    var _getParams = getParams(action),
      params = _extends({}, (_objectDestructuringEmpty(_getParams), _getParams));
    var thisAction = {
      params,
      type: action.type
    };
    set(thisAction, [PARAMS, "lazeInfo"], {
      from: url(),
      time: getTime().toString()
    });
    var lazyAction = getAllLazy();
    if (key) {
      set(lazyAction, [hashKey, key], thisAction);
    } else {
      set(lazyAction, [seqKey], thisAction, true);
    }
    updateLazy(lazyAction);
  };

  /**
   * @template StateType
   * @template ActionType
   * @param {DispatchFunction<StateType, ActionType>} dispatch
   */
  var process = function process(dispatch) {
    return updateLazy(processLazyAction(getAllLazy(), dispatch));
  };

  /**
   * @param {StateType} state
   * @param {ActionObject} action
   */
  var handleAction = function handleAction(state, action) {
    var {
      withLazy
    } = getParams(action);
    if (withLazy) {
      action = getActionMergeWithLazy(action, withLazy);
    }
    var actionHandler = state.get("lazyActionHandler");
    var next = callfunc(actionHandler, [state, action]) || state;
    var {
      wait,
      stop,
      lazyKey
    } = getParams(action); // need locate after next
    if (T_NULL == wait && !stop) {
      if (withLazy && withLazy !== lazyKey) {
        removeLazy(withLazy);
      }
    }
    return next;
  };
  return {
    process,
    handleAction,
    getAll: getAllLazy,
    getOne: getOneLazy,
    push: pushLazyAction
  };
};
export default initLazyAction;