import _classCallCheck from "reshow-runtime/es/helpers/classCallCheck";
import _createClass from "reshow-runtime/es/helpers/createClass";
import _defineProperty from "reshow-runtime/es/helpers/defineProperty";
import { expect } from "chai";
import sinon from "sinon";
import LazyAction from "../LazyAction.mjs";
var FakeMap = /*#__PURE__*/function () {
  function FakeMap() {
    _classCallCheck(this, FakeMap);
    _defineProperty(this, "_state", {});
  }
  return _createClass(FakeMap, [{
    key: "set",
    value: function set(k, v) {
      this._state[k] = v;
    }
  }, {
    key: "get",
    value: function get(k) {
      return this._state[k];
    }
  }]);
}();
describe("Test LazyAction", function () {
  it("test get empty", function () {
    var oMap = new FakeMap();
    var oLazy = LazyAction(oMap);
    var actual = oLazy.getAll();
    expect(actual).to.deep.equal({});
  });
  it("test push lazy", function () {
    var fakeAction = {
      params: {
        foo: "bar"
      }
    };
    var oMap = new FakeMap();
    var oLazy = LazyAction(oMap);
    oLazy.push(fakeAction, "foo");
    var actual = oLazy.getOne("foo");
    expect(actual).to.nested.include({
      "params.foo": "bar"
    });
  });
  it("remove lazy", function () {
    var oMap = new FakeMap();
    var oLazy = LazyAction(oMap);
    oLazy.push({
      params: {
        wait: 999,
        stop: true,
        a: "b",
        lazyKey: "foo"
      }
    }, "foo");
    var laze = oLazy.getOne("foo");
    expect(Object.keys(laze)).to.include("params");
    oLazy.handleAction({
      get: function get() {}
    }, {
      params: {
        withLazy: "foo"
      }
    });
    var afterWithLazy = oLazy.getOne("foo");
    expect(afterWithLazy).to.be.undefined;
  });
});
describe("Test LazyAction Merge", function () {
  it("simple merge", function () {
    var oMap = new FakeMap();
    var oLazy = LazyAction(oMap);
    oLazy.push({
      params: {
        foo: "bar"
      }
    }, "foo");
    var fakeAction = {
      params: {
        abc: "def",
        withLazy: "foo"
      }
    };
    var afterMergeAction;
    oLazy.handleAction({
      get: function get() {
        return function (state, action) {
          afterMergeAction = action;
        };
      }
    }, fakeAction);
    expect(afterMergeAction).to.deep.include({
      params: {
        foo: "bar",
        abc: "def"
      }
    });
  });
  it("complex merge", function () {
    var oMap = new FakeMap();
    var oLazy = LazyAction(oMap);
    oLazy.push({
      params: {
        foo: {
          abc: "def",
          bar: "def"
        }
      }
    }, "foo");
    var fakeAction = {
      params: {
        foo: {
          abc: "bar"
        },
        withLazy: "foo"
      }
    };
    var afterMergeAction;
    oLazy.handleAction({
      get: function get() {
        return function (state, action) {
          afterMergeAction = action;
        };
      }
    }, fakeAction);
    expect(afterMergeAction).to.deep.include({
      params: {
        foo: {
          abc: "bar",
          bar: "def"
        }
      }
    });
  });
  it("with handle stop", function () {
    var oMap = new FakeMap();
    var oLazy = LazyAction(oMap);
    oLazy.push({
      params: {
        wait: 999,
        stop: true,
        a: "b"
      }
    }, "foo");
    var fakeAction = {
      params: {
        withLazy: "foo",
        wait: 777,
        stop: false
      }
    };
    var afterMergeAction;
    oLazy.handleAction({
      get: function get() {
        return function (state, action) {
          afterMergeAction = action;
        };
      }
    }, fakeAction);
    expect(afterMergeAction).to.deep.include({
      params: {
        a: "b",
        wait: 777,
        stop: false
      }
    });
  });
});