import { expect } from "chai";
import { createReducer } from "reshow-flux-base";
import { SimpleMap } from "reshow-map";
import sinon from "sinon";
import BaseI13nReducer from "../BaseI13nReducer.mjs";
import i13nStoreReAssign from "../../i13nStoreReAssign.mjs";
import heeding from "../../heeding.mjs";
describe("Test BaseI13nReducer", function () {
  var oI13n = new BaseI13nReducer();
  var [i13nStore, i13nDispatch] = createReducer(oI13n.reduce.bind(oI13n), new SimpleMap());
  i13nStoreReAssign({
    oI13n,
    store: i13nStore,
    i13nDispatch,
    mergeMap: function mergeMap(state, jsArr) {
      return state.merge(jsArr);
    }
  });
  beforeEach(function (done) {
    i13nDispatch("reset");
    setTimeout(function () {
      done();
    }, 50);
  });
  it("test init", function (done) {
    var impressionHandler = sinon.spy(function (state) {
      return state;
    });
    var initHandler = sinon.spy(function (state, action, done) {
      done(state);
      return state;
    });
    i13nDispatch("config/set", {
      impressionHandler,
      initHandler
    });
    expect(!!i13nStore.getState().get("init")).to.be.false;
    i13nDispatch("impression");
    expect(initHandler.called).to.be.true;
    setTimeout(function () {
      expect(i13nStore.getState().get("init")).to.be.true;
      expect(initHandler.callCount).to.equal(1);
      i13nDispatch("impression");
      expect(initHandler.callCount).to.equal(1);
      setTimeout(function () {
        expect(impressionHandler.callCount).to.equal(2);
        done();
      }, 50);
    });
  });
  it("test dispatch action", function (done) {
    var actionHandler = sinon.spy(function (state) {
      return state;
    });
    var afterAction = sinon.spy(function (state, action) {});
    i13nStore.addListener(heeding(afterAction, "action"));
    i13nDispatch("config/set", {
      actionHandler
    });
    expect(actionHandler.called).to.be.false;
    i13nDispatch("action");
    setTimeout(function () {
      expect(actionHandler.called).to.be.true;
      expect(afterAction.callCount).to.equal(1);
      done();
    });
  });
  it("test dispatch impression", function (done) {
    var impressionHandler = sinon.spy(function (state) {
      return state;
    });
    var afterImpression = sinon.spy(function (state, action) {});
    i13nStore.addListener(heeding(afterImpression, "impression"));
    i13nDispatch("config/set", {
      impressionHandler
    });
    expect(impressionHandler.called).to.be.false;
    i13nDispatch("impression");
    setTimeout(function () {
      expect(impressionHandler.called).to.be.true;
      expect(afterImpression.callCount).to.equal(1);
      done();
    }, 10);
  });
});