import _classCallCheck from "reshow-runtime/es/helpers/classCallCheck";
import _createClass from "reshow-runtime/es/helpers/createClass";
import _defineProperty from "reshow-runtime/es/helpers/defineProperty";
import { expect } from "chai";
import sinon from "sinon";
import { createReducer } from "reshow-flux-base";
import { SimpleMap } from "reshow-map";
import BaseI13nReducer from "../BaseI13nReducer.mjs";
import LazyAction from "../../LazyAction.mjs";
import i13nStoreReAssign from "../../i13nStoreReAssign.mjs";
import heeding from "../../heeding.mjs";
var FakeMap = /*#__PURE__*/function () {
  function FakeMap() {
    _classCallCheck(this, FakeMap);
    _defineProperty(this, "_state", {});
  }
  return _createClass(FakeMap, [{
    key: "set",
    value: function set(k, v) {
      this._state[k] = v;
    }
  }, {
    key: "get",
    value: function get(k) {
      return this._state[k];
    }
  }]);
}();
describe("Test LazyAction with flux", function () {
  it("test with flux-action", function (done) {
    var oMap = new FakeMap();
    var oLazy = LazyAction(oMap);
    var oI13n = new BaseI13nReducer();
    var [i13nStore, i13nDispatch] = createReducer(oI13n.reduce.bind(oI13n), new SimpleMap());
    i13nStoreReAssign({
      oI13n,
      store: i13nStore,
      i13nDispatch,
      mergeMap: function mergeMap(state, jsArr) {
        return state.merge(jsArr);
      }
    });
    i13nDispatch("reset", {
      actionHandler: oLazy.handleAction,
      lazyActionHandler: function lazyActionHandler(state, action) {
        expect(action).to.deep.equal({
          params: {
            a: "b",
            wait: 777,
            stop: false
          },
          type: "action"
        });
        state = state.set('thisAction', action);
        return state;
      }
    });
    var spy = sinon.spy(function () {
      return null;
    });
    i13nStore.addListener(spy);
    oLazy.push({
      params: {
        wait: 999,
        stop: true,
        a: "b"
      }
    }, "foo");
    i13nDispatch("action", {
      withLazy: "foo",
      wait: 777,
      stop: false
    });
    setTimeout(function () {
      expect(spy.callCount, "[Action call]").to.equal(1);
      done();
    });
  });
  it("test with flux-impression", function (done) {
    var oMap = new FakeMap();
    var oLazy = LazyAction(oMap);
    var oI13n = new BaseI13nReducer();
    var [i13nStore, i13nDispatch] = createReducer(oI13n.reduce.bind(oI13n), new SimpleMap());
    i13nStoreReAssign({
      oI13n,
      store: i13nStore,
      i13nDispatch,
      mergeMap: function mergeMap(state, jsArr) {
        return state.merge(jsArr);
      }
    });
    i13nDispatch("reset", {
      initHandler: function initHandler(state, action, initDone) {
        oLazy.process(i13nDispatch);
        state = state.set('foo', 'bar');
        initDone(state, action);
        return state;
      }
    });
    var spyInit = sinon.spy();
    var spyImpression = sinon.spy();
    i13nStore.addListener(heeding(spyInit, "init"));
    i13nStore.addListener(heeding(spyImpression, "impression"));
    i13nDispatch("impression");
    setTimeout(function () {
      expect(spyInit.callCount, "[Init Call]").to.equal(1);
      expect(spyImpression.callCount, "[Impression Call]").to.equal(1);
      done();
    }, 30);
  });
});